@@ -99,14 +99,17 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_winconfig.h 3092 2006-01-28 07:57:37Z fyodor $ */
+/* $Id: nmap_winconfig.h 3575 2006-06-24 04:13:43Z fyodor $ */
 
 #ifndef NMAP_WINCONFIG_H
 #define NMAP_WINCONFIG_H
-
-#define NMAP_VERSION "4.00"
+/* Without this, Windows will give us all sorts of crap about using functions
+   like strcpy() even if they are done safely */
+#define _CRT_SECURE_NO_DEPRECATE 1
+#define NMAP_VERSION "4.11"
 #define NMAP_NAME "Nmap"
 #define NMAP_URL "http://www.insecure.org/nmap"
 #define NMAP_PLATFORM "i686-pc-windows-windows"
 #define NMAPDATADIR "c:\\nmap" /* FIXME: I really need to make this dynamic */
+
 #endif /* NMAP_WINCONFIG_H */
