@@ -56,7 +56,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_iod.c 2396 2004-08-29 09:12:05Z fyodor $ */
+/* $Id: nsock_iod.c 3439 2006-06-10 21:23:36Z fyodor $ */
 
 #include "nsock.h"
 #include "nsock_internal.h"
@@ -95,7 +95,10 @@ nsock_iod nsi_new2(nsock_pool nsockp, int sd, void *userdata) {
     nsi->state = NSIOD_STATE_INITIAL;
   } else {
     nsi->sd = dup(sd);
-    if (nsi->sd == -1) return NULL;
+    if (nsi->sd == -1) {
+      free(nsi);
+      return NULL;
+    }
     nsock_unblock_socket(nsi->sd);
     nsi->state = NSIOD_STATE_UNKNOWN;
   }
