@@ -54,7 +54,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_write.c 2396 2004-08-29 09:12:05Z fyodor $ */
+/* $Id: nsock_write.c 3439 2006-06-10 21:23:36Z fyodor $ */
 
 #include "nsock.h"
 #include "nsock_internal.h"
@@ -125,7 +125,6 @@ nsock_event_id nsock_printf(nsock_pool ms_pool, nsock_iod ms_iod,
   res = vsnprintf(buf, sizeof(buf), format, ap);
   va_end(ap);
 
-
   if (res != -1) {
     if (res > sizeof(buf)) {
       buf2 = (char * ) safe_malloc(res + 16);
@@ -168,6 +167,10 @@ nsock_event_id nsock_printf(nsock_pool ms_pool, nsock_iod ms_iod,
 		  nsi->id, nse->id, displaystr);
   }
 
+  if (buf2 != buf) {
+    free(buf2);
+  }
+
   nsp_add_event(nsp, nse);
 
   return nse->id;
