@@ -101,7 +101,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: output.cc 3048 2006-01-19 07:29:12Z fyodor $ */
+/* $Id: output.cc 3408 2006-05-31 23:01:19Z fyodor $ */
 
 #include "output.h"
 #include "osscan.h"
@@ -116,7 +116,45 @@ namespace std {};
 using namespace std;
 
 extern NmapOps o;
-static char *logtypes[LOG_TYPES]=LOG_NAMES;
+static char *logtypes[LOG_NUM_FILES]=LOG_NAMES;
+
+/* Used in creating skript kiddie style output.  |<-R4d! */
+static void skid_output(char *s)
+{
+  int i;
+  for (i=0;s[i];i++)
+    if (rand()%2==0)
+      /* Substitutions commented out are not known to me, but maybe look nice */
+      switch(s[i])
+	{
+	case 'A': s[i]='4'; break;
+	  /*	case 'B': s[i]='8'; break;
+	 	case 'b': s[i]='6'; break;
+	        case 'c': s[i]='k'; break;
+	        case 'C': s[i]='K'; break; */
+	case 'e':
+	case 'E': s[i]='3'; break;
+	case 'i':
+	case 'I': s[i]="!|1"[rand()%3]; break;
+	  /*      case 'k': s[i]='c'; break;
+	        case 'K': s[i]='C'; break;*/
+	case 'o':
+	case 'O': s[i]='0'; break;
+	case 's':
+	case 'S': 
+	  if (s[i+1] && !isalnum((int) s[i+1])) 
+	    s[i] = 'z';
+	  else s[i] = '$';
+	  break;
+	case 'z': s[i]='s'; break;
+	case 'Z': s[i]='S'; break;
+	}  
+    else
+      {
+	if (s[i]>='A' && s[i]<='Z' && (rand()%3==0)) s[i]+='a'-'A';
+	else if (s[i]>='a' && s[i]<='z' && (rand()%3==0)) s[i]-='a'-'A';
+      }
+}
 
 
 // Creates an XML <service> element for the information given in
@@ -128,10 +166,11 @@ static int getServiceXMLBuf(struct serviceDeductions *sd, char *xmlbuf,
   char rpcbuf[128];
   char *xml_product = NULL, *xml_version = NULL, *xml_extrainfo = NULL;
   char *xml_hostname = NULL, *xml_ostype = NULL, *xml_devicetype = NULL;
+  char *xml_servicefp = NULL, *xml_servicefp_temp = NULL;
 
   if (xmlbuflen < 1) return -1;
-  xmlbuf[0] = '\0';
-  if (!sd->name) return 0;
+  xmlbuf[0] = '\0';  
+  if (!sd->name && !sd->service_fp) return 0;
 
   if (sd->product) {
     xml_product = xml_convert(sd->product);
@@ -181,6 +220,16 @@ static int getServiceXMLBuf(struct serviceDeductions *sd, char *xmlbuf,
     versionxmlstring += '\"';
   }
 
+  if (sd->service_fp) {
+    xml_servicefp_temp = xml_convert(sd->service_fp);
+    xml_servicefp = xml_sf_convert(xml_servicefp_temp);
+    versionxmlstring += " servicefp=\"";
+    versionxmlstring += xml_servicefp;
+    free(xml_servicefp_temp); xml_servicefp_temp = NULL;
+    free(xml_servicefp); xml_servicefp = NULL;
+    versionxmlstring += '\"';
+  }
+
   if (o.rpcscan && sd->rpc_status == RPC_STATUS_GOOD_PROG) {
     snprintf(rpcbuf, sizeof(rpcbuf), 
 	     " rpcnum=\"%li\" lowver=\"%i\" highver=\"%i\" proto=\"rpc\"", 
@@ -189,7 +238,7 @@ static int getServiceXMLBuf(struct serviceDeductions *sd, char *xmlbuf,
 
   snprintf(xmlbuf, xmlbuflen, 
 	   "<service name=\"%s\"%s %smethod=\"%s\" conf=\"%d\"%s />", 
-	   sd->name,
+	   sd->name? sd->name : "unknown",
 	   versionxmlstring.c_str(),
 	   (sd->service_tunnel == SERVICE_TUNNEL_SSL)? "tunnel=\"ssl\" " : "",
 	   (sd->dtype == SERVICE_DETECTION_TABLE)? "table" : "probed", 
@@ -319,7 +368,7 @@ void printportoutput(Target *currenths, PortList *plist) {
   char rpcinfo[64];
   char rpcmachineinfo[64];
   char portinfo[64];
-  char xmlbuf[512];
+  char xmlbuf[2560];
   char grepvers[256];
   char grepown[64];
   char *p;
@@ -330,11 +379,7 @@ void printportoutput(Target *currenths, PortList *plist) {
   int first = 1;
   struct protoent *proto;
   Port *current;
-  int numignoredports;
-  int portno;
   char hostname[1200];
-  int istate = plist->getIgnoredPortState();
-  numignoredports = plist->state_counts[istate];
   struct serviceDeductions sd;
   NmapOutputTable *Tbl = NULL;
   int portcol = -1; // port or IP protocol #
@@ -345,23 +390,38 @@ void printportoutput(Target *currenths, PortList *plist) {
   int colno = 0;
   unsigned int rowno;
   int numrows;
+  int numignoredports = plist->numIgnoredPorts();
+
   vector<const char *> saved_servicefps;
 
-  //cout << numignoredports << " " << plist->numports << endl;
-  assert(numignoredports <= plist->numports);
-
-
-  log_write(LOG_XML, "<ports><extraports state=\"%s\" count=\"%d\" />\n", 
-	    statenum2str(istate), 
-	    numignoredports);
+  log_write(LOG_XML, "<ports>");
+  int prevstate = PORT_UNKNOWN;
+  int istate;
+  while ((istate = plist->nextIgnoredState(prevstate)) != PORT_UNKNOWN) {
+    log_write(LOG_XML, "<extraports state=\"%s\" count=\"%d\" />\n", 
+	    statenum2str(istate), plist->getStateCounts(istate));
+    prevstate = istate;
+  }
 
   if (numignoredports == plist->numports) {
     log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT,
-              "%s %d scanned %s on %s %s: %s\n",
+              "%s %d scanned %s on %s %s ",
 	      (numignoredports == 1)? "The" : "All", numignoredports,
 	      (numignoredports == 1)? "port" : "ports", 
 	      currenths->NameIP(hostname, sizeof(hostname)), 
-	      (numignoredports == 1)? "is" : "are", statenum2str(istate));
+	      (numignoredports == 1)? "is" : "are");
+    if (plist->numIgnoredStates() == 1) {
+      log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT, statenum2str(plist->nextIgnoredState(PORT_UNKNOWN)));
+    } else {
+      prevstate = PORT_UNKNOWN;
+      while ((istate = plist->nextIgnoredState(prevstate)) != PORT_UNKNOWN) {
+	if (prevstate != PORT_UNKNOWN) log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT, " or ");
+	log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT, "%s (%d)", statenum2str(istate), plist->getStateCounts(istate));
+	prevstate = istate;
+      }
+    }
+    log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT, "\n");
+
     log_write(LOG_MACHINE,"Host: %s (%s)\tStatus: Up", 
 	      currenths->targetipstr(), currenths->HostName());
     log_write(LOG_XML, "</ports>\n");
@@ -374,9 +434,18 @@ void printportoutput(Target *currenths, PortList *plist) {
   log_write(LOG_MACHINE,"Host: %s (%s)", currenths->targetipstr(), 
 	    currenths->HostName());
   
-  if (numignoredports > 0) {
-    log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT,"(The %d %s%s scanned but not shown below %s in state: %s)\n", numignoredports, o.ipprotscan?"protocol":"port", (numignoredports == 1)? "" : "s", (numignoredports == 1)? "is" : "are", statenum2str(istate));
+  /* Show line like:
+     Not shown: 3995 closed ports, 514 filtered ports
+     if appropriate (note that states are reverse-sorted by # of ports) */
+  prevstate = PORT_UNKNOWN;
+  while ((istate = plist->nextIgnoredState(prevstate)) != PORT_UNKNOWN) {
+    if (prevstate == PORT_UNKNOWN) 
+      log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT, "Not shown: ");
+    else log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT, ", ");
+    log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT, "%d %s %s", plist->getStateCounts(istate), statenum2str(istate), o.ipprotscan? "protocols": "ports");
+    prevstate = istate;
   }
+  if (prevstate != PORT_UNKNOWN) log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT, "\n");
 
   /* OK, now it is time to deal with the service table ... */
   colno = 0;
@@ -388,13 +457,8 @@ void printportoutput(Target *currenths, PortList *plist) {
   if (o.servicescan || o.rpcscan)
     versioncol = colno++;
 
-  numrows = plist->state_counts[PORT_CLOSED] + 
-    plist->state_counts[PORT_OPEN] + plist->state_counts[PORT_FILTERED] + 
-    plist->state_counts[PORT_UNFILTERED] + 
-    plist->state_counts[PORT_OPENFILTERED] + 
-    plist->state_counts[PORT_CLOSEDFILTERED];
-  if (istate != PORT_UNKNOWN)
-    numrows -=  plist->state_counts[istate];
+  numrows = plist->numports - numignoredports;
+
   assert(numrows > 0);
   numrows++; // The header counts as a row
 
@@ -414,20 +478,18 @@ void printportoutput(Target *currenths, PortList *plist) {
 
   log_write(LOG_MACHINE,"\t%s: ", (o.ipprotscan)? "Protocols" : "Ports" );
   
-  current = NULL;
   rowno = 1;
   if (o.ipprotscan) {
-    for (portno = 0; portno < 256; portno++) {
-      if (!plist->ip_prots[portno]) continue;
-      current = plist->ip_prots[portno];
-      if (current->state != istate) {
+    current = NULL;
+    while( (current=plist->nextPort(current, IPPROTO_IP, 0))!=NULL ) {
+      if (!plist->isIgnoredState(current->state)) {
 	if (!first) log_write(LOG_MACHINE,", ");
 	else first = 0;
 	state = statenum2str(current->state);
 	proto = nmap_getprotbynum(htons(current->portno));
 	snprintf(portinfo, sizeof(portinfo), "%-24s",
 		 proto?proto->p_name: "unknown");
-	Tbl->addItemFormatted(rowno, portcol, "%d", portno);
+	Tbl->addItemFormatted(rowno, portcol, "%d", current->portno);
 	Tbl->addItem(rowno, statecol, true, state);
 	Tbl->addItem(rowno, servicecol, true, portinfo);
 	log_write(LOG_MACHINE,"%d/%s/%s/", current->portno, state, 
@@ -440,21 +502,9 @@ void printportoutput(Target *currenths, PortList *plist) {
       }
     }
   } else {
-    map<u16,Port*>::iterator tcpIter = plist->tcp_ports.begin();
-    map<u16,Port*>::iterator udpIter = plist->udp_ports.begin();
-
-    while (tcpIter != plist->tcp_ports.end() || udpIter != plist->udp_ports.end()) {
-
-      // If the udp iterator is at the end, then we always read from tcp and vica-versa
-      if (tcpIter != plist->tcp_ports.end() && (udpIter == plist->udp_ports.end() || tcpIter->first <= udpIter->first)) {
-	current = tcpIter->second;
-	tcpIter++;
-      } else {
-	current = udpIter->second;
-	udpIter++;
-      }
-
-      if (current->state != istate) {    
+    current = NULL;
+    while( (current=plist->nextPort(current, TCPANDUDP, 0))!=NULL ) {
+      if (!plist->isIgnoredState(current->state)) {
 	if (!first) log_write(LOG_MACHINE,", ");
 	else first = 0;
 	strcpy(protocol,(current->proto == IPPROTO_TCP)? "tcp": "udp");
@@ -556,8 +606,8 @@ void printportoutput(Target *currenths, PortList *plist) {
     
   }
   /*  log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT,"\n"); */
-  if (plist->state_counts[istate] > 0)
-    log_write(LOG_MACHINE, "\tIgnored State: %s (%d)", statenum2str(istate), plist->state_counts[istate]);
+  if (plist->getStateCounts(istate) > 0)
+    log_write(LOG_MACHINE, "\tIgnored State: %s (%d)", statenum2str(istate), plist->getStateCounts(istate));
   log_write(LOG_XML, "</ports>\n");
 
   // Now we write the table for the user
@@ -617,53 +667,135 @@ char* xml_convert (const char* str) {
   return temp;
 }
 
+/* This is the workhorse of the logging functions.  Usually it is
+   called through log_write(), but it can be called directly if you
+   are dealing with a vfprintf-style va_list.  Unlike log_write, YOU
+   CAN ONLY CALL THIS WITH ONE LOG TYPE (not a bitmask full of them).
+   In addition, YOU MUST SANDWHICH EACH EXECUTION IF THIS CALL BETWEEN
+   va_start() AND va_end() calls. */
+void log_vwrite(int logt, const char *fmt, va_list ap) {
+  static char *writebuf = NULL;
+  int writebuflen = 65536;
+  bool skid_noxlate = false;
+  int rc = 0;
+  int len;
+  int fileidx = 0;
+  int l;
+
+  /* Account for extended output under high debugging/verbosity */
+  if (o.debugging > 2 || o.verbose > 2)
+    writebuflen *= 8;
+
+  if (!writebuf)
+    writebuf = (char *) safe_malloc(writebuflen);
+
+  if (logt == LOG_SKID_NOXLT) {
+    logt = LOG_SKID;
+    skid_noxlate = true;
+  }
+
+  switch(logt) {
+  case LOG_STDOUT: 
+    vfprintf(o.nmap_stdout, fmt, ap);
+    break;
+
+  case LOG_STDERR: 
+    fflush(stdout); // Otherwise some systems will print stderr out of order
+    vfprintf(stderr, fmt, ap);
+    break;
+
+  case LOG_NORMAL:
+  case LOG_MACHINE:
+  case LOG_SKID:
+  case LOG_XML:
+    l = logt;
+    fileidx = 0;
+    while ((l&1)==0) { fileidx++; l>>=1; }
+    assert(fileidx < LOG_NUM_FILES);
+    if (o.logfd[fileidx]) {
+      len = vsnprintf(writebuf, writebuflen, fmt, ap);
+      if (len == 0) { 
+	return;
+      } else if (len < 0) {
+	fprintf(stderr, "vnsprintf returned %d in %s -- bizarre. Quitting.\n", len, __FUNCTION__);
+	exit(1);
+      } else if (len >= writebuflen) {
+	fprintf(stderr, "%s: write buffer not large enough -- need to increase from %d to at least %d (logt == %d).  Please email this message to fyodor@insecure.org.  Quitting.\n", __FUNCTION__, writebuflen, len, logt);
+	exit(1);
+      }
+      if (logt == LOG_SKID && !skid_noxlate)
+	skid_output(writebuf);
+      rc = fwrite(writebuf,len,1,o.logfd[fileidx]);
+      if (rc != 1) {
+	fprintf(stderr, "Failed to write %d bytes of data to (logt==%d) stream. fwrite returned %d.  Quitting.\n", len, logt, rc);
+	exit(1);
+      }
+    }
+    break;
+
+  default:
+    fprintf(stderr, "log_vwrite(): Passed unknown log type (%d).  Note that this function, unlike log_write, can only handle one log type at a time (no bitmasks)\n", logt);
+    exit(1);
+  }
+
+  return;
+}
+
+/* Remove all "\nSF:" from fingerprints */
+char* xml_sf_convert (const char* str) {
+  char *temp = (char *) safe_malloc(strlen(str) + 1);
+  char *dst = temp, *src = (char *)str;
+  char *ampptr = 0;
+  int charcount = 0;
+
+  while(*src && charcount < 2035) { /* 2048 - 14 */
+    if (strncmp(src, "\nSF:", 4) == 0) {
+      src += 4;
+      continue;
+    }
+    /* Needed so "&something;" is not truncated midway */
+    if (*src == '&') {
+      ampptr = dst;
+    }
+    else if (*src == ';') {
+      ampptr = 0;
+    }
+    *dst++ = *src++;
+    charcount++;
+  }
+  if (ampptr != 0) {
+    *ampptr = '\0';
+  }
+  else {
+    *dst = '\0';
+  }
+  return temp;
+}
+
 /* Write some information (printf style args) to the given log stream(s).
  Remember to watch out for format string bugs.  */
 void log_write(int logt, const char *fmt, ...)
 {
   va_list  ap;
-  int i,l=logt,skid=1;
-  char b[4096];
-  char *buf = b;
-  int bufsz = sizeof(b);
-  bool buf_alloced = false;
-  int rc = 0;
+  assert(logt > 0);
 
-  if (l & LOG_STDOUT) {
-    va_start(ap, fmt);
-    vfprintf(o.nmap_stdout, fmt, ap);
-    va_end(ap);
-    l-=LOG_STDOUT;
-  }
-  if (l & LOG_SKID_NOXLT) { skid=0; l -= LOG_SKID_NOXLT; l |= LOG_SKID; }
-  if (l<0 || l>LOG_MASK) return;
-  for (i=0;l;l>>=1,i++)
-    {
-      if (!o.logfd[i] || !(l&1)) continue;
-      while(1) {
-	va_start(ap, fmt);
-	rc = vsnprintf(buf,bufsz, fmt, ap);
-	va_end(ap);
-	if (rc >= 0 && rc < bufsz)
-	  break; // Successful
-	// D'oh!  Apparently not enough space - lets try a bigger buffer
-	bufsz = (rc > bufsz)? rc + 1 : bufsz * 2;
-	buf = (char *) safe_realloc(buf_alloced? buf : NULL, bufsz);
-	buf_alloced = true;
-      } 
-      if (skid && ((1<<i)&LOG_SKID)) skid_output(buf);
-      fwrite(buf,1,strlen(buf),o.logfd[i]);
+  if (!fmt || !*fmt) return;
+
+  for (int l = 1; l <= LOG_MAX; l <<= 1) {
+    if (logt & l) {
+      va_start(ap, fmt);
+      log_vwrite(l, fmt, ap);
+      va_end(ap);
     }
-
-  if (buf_alloced)
-    free(buf);
+  }
+  return;
 }
 
 /* Close the given log stream(s) */
 void log_close(int logt)
 {
   int i;
-  if (logt<0 || logt>LOG_MASK) return;
+  if (logt<0 || logt>LOG_FILE_MASK) return;
   for (i=0;logt;logt>>=1,i++) if (o.logfd[i] && (logt&1)) fclose(o.logfd[i]);
 }
 
@@ -676,10 +808,16 @@ void log_flush(int logt) {
     fflush(o.nmap_stdout);
     logt -= LOG_STDOUT;
   }
+
+  if (logt & LOG_STDERR) {
+    fflush(stderr);
+    logt -= LOG_STDERR;
+  }
+
   if (logt & LOG_SKID_NOXLT)
     fatal("You are not allowed to log_flush() with LOG_SKID_NOXLT");
 
-  if (logt<0 || logt>LOG_MASK) return;
+  if (logt<0 || logt>LOG_FILE_MASK) return;
 
   for (i=0;logt;logt>>=1,i++)
     {
@@ -694,7 +832,7 @@ void log_flush(int logt) {
 void log_flush_all() {
   int fileno;
 
-  for(fileno = 0; fileno < LOG_TYPES; fileno++) {
+  for(fileno = 0; fileno < LOG_NUM_FILES; fileno++) {
     if (o.logfd[fileno]) fflush(o.logfd[fileno]);
   }
   fflush(stdout);
@@ -707,7 +845,7 @@ void log_flush_all() {
 int log_open(int logt, int append, char *filename)
 {
   int i=0;
-  if (logt<=0 || logt>LOG_MASK) return -1;
+  if (logt<=0 || logt>LOG_FILE_MASK) return -1;
   while ((logt&1)==0) { i++; logt>>=1; }
   if (o.logfd[i]) fatal("Only one %s output filename allowed",logtypes[i]);
   if (*filename == '-' && *(filename + 1) == '\0')
@@ -729,48 +867,11 @@ int log_open(int logt, int append, char *filename)
   return 1;
 }
 
-/* Used in creating skript kiddie style output.  |<-R4d! */
-void skid_output(char *s)
-{
-  int i;
-  for (i=0;s[i];i++)
-    if (rand()%2==0)
-      /* Substitutions commented out are not known to me, but maybe look nice */
-      switch(s[i])
-	{
-	case 'A': s[i]='4'; break;
-	  /*	case 'B': s[i]='8'; break;
-	 	case 'b': s[i]='6'; break;
-	        case 'c': s[i]='k'; break;
-	        case 'C': s[i]='K'; break; */
-	case 'e':
-	case 'E': s[i]='3'; break;
-	case 'i':
-	case 'I': s[i]="!|1"[rand()%3]; break;
-	  /*      case 'k': s[i]='c'; break;
-	        case 'K': s[i]='C'; break;*/
-	case 'o':
-	case 'O': s[i]='0'; break;
-	case 's':
-	case 'S': 
-	  if (s[i+1] && !isalnum((int) s[i+1])) 
-	    s[i] = 'z';
-	  else s[i] = '$';
-	  break;
-	case 'z': s[i]='s'; break;
-	case 'Z': s[i]='S'; break;
-	}  
-    else
-      {
-	if (s[i]>='A' && s[i]<='Z' && (rand()%3==0)) s[i]+='a'-'A';
-	else if (s[i]>='a' && s[i]<='z' && (rand()%3==0)) s[i]-='a'-'A';
-      }
-}
 
 /* The items in ports should be
    in sequential order for space savings and easier to read output.  Outputs
    the rangelist to the log stream given (such as LOG_MACHINE or LOG_XML) */
-void output_rangelist_given_ports(int logt, unsigned short *ports,
+static void output_rangelist_given_ports(int logt, unsigned short *ports,
 						    int numports) {
 int i, previous_port = -2, range_start = -2, port;
 char outpbuf[128];
@@ -786,7 +887,8 @@ char outpbuf[128];
 	 strcat(outpbuf, ",");
        sprintf(outpbuf + strlen(outpbuf), "%hu", port);
      }
-     log_write(logt, "%s", outpbuf);
+     if (*outpbuf)
+       log_write(logt, "%s", outpbuf);
      range_start = port;
    }
    previous_port = port;
@@ -852,6 +954,26 @@ void output_xml_scaninfo_records(struct scan_lists *scanlist) {
   log_flush_all();
 }
 
+/* Prints the MAC address (if discovered) to XML output */
+static void print_MAC_XML_Info(Target *currenths) {
+  const u8 *mac = currenths->MACAddress();
+  char macascii[32];
+  char vendorstr[128];
+  char *xml_mac = NULL;
+
+  if (mac) {
+    const char *macvendor = MACPrefix2Corp(mac);
+    snprintf(macascii, sizeof(macascii), "%02X:%02X:%02X:%02X:%02X:%02X",
+	     mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
+    if (macvendor) {
+      xml_mac = xml_convert(macvendor);
+      snprintf(vendorstr, sizeof(vendorstr), " vendor=\"%s\"", xml_mac);
+      free(xml_mac);
+    } else vendorstr[0] = '\0';
+    log_write(LOG_XML, "<address addr=\"%s\" addrtype=\"mac\"%s />\n", macascii, vendorstr);
+  }
+}
+
 /* Helper function to write the status and address/hostname info of a host 
    into the XML log */
 static void write_xml_initial_hostinfo(Target *currenths,
@@ -1084,25 +1206,6 @@ void printmacinfo(Target *currenths) {
   }
 }
 
-/* Prints the MAC address (if discovered) to XML output */
-void print_MAC_XML_Info(Target *currenths) {
-  const u8 *mac = currenths->MACAddress();
-  char macascii[32];
-  char vendorstr[128];
-  char *xml_mac = NULL;
-
-  if (mac) {
-    const char *macvendor = MACPrefix2Corp(mac);
-    snprintf(macascii, sizeof(macascii), "%02X:%02X:%02X:%02X:%02X:%02X",
-	     mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
-    if (macvendor) {
-      xml_mac = xml_convert(macvendor);
-      snprintf(vendorstr, sizeof(vendorstr), " vendor=\"%s\"", xml_mac);
-      free(xml_mac);
-    } else vendorstr[0] = '\0';
-    log_write(LOG_XML, "<address addr=\"%s\" addrtype=\"mac\"%s />\n", macascii, vendorstr);
-  }
-}
 
 
 /* Prints the formatted OS Scan output to stdout, logfiles, etc (but only
@@ -1304,7 +1407,7 @@ void printserviceinfooutput(Target *currenths) {
   for (i=0; i<MAX_SERVICE_INFO_FIELDS; i++)
     hostname_tbl[i][0] = ostype_tbl[i][0] = devicetype_tbl[i][0] = '\0';
 
-  while ((p = currenths->ports.nextPort(p, 0, PORT_OPEN, false))) {
+  while ((p = currenths->ports.nextPort(p, TCPANDUDP, PORT_OPEN))) {
     // The following 2 lines (from portlist.h) tell us that we don't
     // need to worry about free()ing anything in the serviceDeductions struct.
       // pass in an allocated struct serviceDeductions (don't wory about initializing, and
@@ -1394,8 +1497,8 @@ void printStatusMessage() {
   
   log_write(LOG_STDOUT, 
 	    "Stats: %d:%02d:%02d elapsed; %d hosts completed (%d up), %d undergoing %s\n", 
-	    time/60/24, time/60 % 24, time % 60, o.numhosts_scanned, 
-	    o.numhosts_up, o.numhosts_scanning, scantype2str(o.scantype));
+	    time/60/24, time/60 % 24, time % 60, o.numhosts_scanned - o.numhosts_scanning, 
+	    o.numhosts_up, o.numhosts_scanning, scantype2str(o.current_scantype));
 }
 
 
