@@ -101,22 +101,23 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: output.h 2820 2005-08-24 19:22:11Z fyodor $ */
+/* $Id: output.h 3274 2006-04-22 23:00:09Z fyodor $ */
 
 #ifndef OUTPUT_H
 #define OUTPUT_H
 
-#define LOG_TYPES 5
-#define LOG_MASK 31
+#define LOG_NUM_FILES 4 /* # of values that actual files (they must come first */
+#define LOG_FILE_MASK 15 /* The mask for log typs in the file array */
 #define LOG_NORMAL 1
 #define LOG_MACHINE 2
-#define LOG_HTML 4
-#define LOG_SKID 8
-#define LOG_XML 16
+#define LOG_SKID 4
+#define LOG_XML 8
 #define LOG_STDOUT 1024
-#define LOG_SKID_NOXLT 2048
+#define LOG_STDERR 2048
+#define LOG_SKID_NOXLT 4096
+#define LOG_MAX LOG_SKID_NOXLT /* The maximum log type value */
 
-#define LOG_NAMES {"normal", "machine", "HTML", "$Cr!pT |<!dd!3", "XML"}
+#define LOG_NAMES {"normal", "machine", "$Cr!pT |<!dd!3", "XML"}
 
 #include "portlist.h"
 #include "tcpip.h"
@@ -135,14 +136,19 @@ void printportoutput(Target *currenths, PortList *plist);
    in a certain place to conform to DTD. */
 void printmacinfo(Target *currenths);
 
-/* Prints the MAC address (if discovered) to XML output */
-void print_MAC_XML_Info(Target *currenths);
-
 /* Write some information (printf style args) to the given log stream(s).
    Remember to watch out for format string bugs. */
 void log_write(int logt, const char *fmt, ...)
      __attribute__ ((format (printf, 2, 3)));
 
+/* This is the workhorse of the logging functions.  Usually it is
+   called through log_write(), but it can be called directly if you
+   are dealing with a vfprintf-style va_list.  Unlike log_write, YOU
+   CAN ONLY CALL THIS WITH ONE LOG TYPE (not a bitmask full of them).
+   In addition, YOU MUST SANDWHICH EACH EXECUTION IF THIS CALL BETWEEN
+   va_start() AND va_end() calls. */
+void log_vwrite(int logt, const char *fmt, va_list ap);
+
 /* Close the given log stream(s) */
 void log_close(int logt);
 
@@ -159,9 +165,6 @@ void log_flush_all();
    it already exists.  If the file does not exist, it will be created */
 int log_open(int logt, int append, char *filename);
 
-/* Used in creating skript kiddie style output.  |<-R4d! */
-void skid_output(char *s);
-
 /* Output the list of ports scanned to the top of machine parseable
    logs (in a comment, unfortunately).  The items in ports should be
    in sequential order for space savings and easier to read output */
@@ -169,13 +172,6 @@ void output_ports_to_machine_parseable_output(struct scan_lists *ports,
 					      int tcpscan, int udpscan,
 					      int protscan);
 
-/* The items in ports should be
-   in sequential order for space savings and easier to read output.  Outputs
-   the rangelist to the log stream given (such as LOG_MACHINE or LOG_XML) */
-void output_rangelist_given_ports(int logt, unsigned short *ports,
-				  int numports);
-
-
 /* Similar to output_ports_to_machine_parseable_output, this function
    outputs the XML version, which is scaninfo records of each scan
    requested and the ports which it will scan for */
@@ -199,10 +195,13 @@ void printserviceinfooutput(Target *currenths);
    normal/skiddy/stdout output */
 int print_iflist(void);
 
+/* Prints a status message while the program is running */
+void printStatusMessage();
+
 /* Prints the statistics and other information that goes at the very end
    of an Nmap run */
-void printfinaloutput(int numhosts_scanned, int numhosts_up, 
-		      time_t starttime);
+void printfinaloutput();
 
 char* xml_convert (const char* str);
+char* xml_sf_convert (const char* str);
 #endif /* OUTPUT_H */
