@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: service_scan.cc 3201 2006-03-06 00:15:49Z fyodor $ */
+/* $Id: service_scan.cc 3516 2006-06-19 04:19:52Z fyodor $ */
 
 
 #include "service_scan.h"
@@ -851,11 +851,6 @@ ServiceProbe::~ServiceProbe() {
   if (fallbackStr) free(fallbackStr);
 }
 
-void ServiceProbe::setName(const char *name) {
-  if (probename) free(probename);
-  probename = strdup(name);
-}
-
   // Parses the "probe " line in the nmap-service-probes file.  Pass the rest of the line
   // after "probe ".  The format better be:
   // [TCP|UDP] [probename] q|probetext|
@@ -1043,7 +1038,9 @@ void ServiceProbe::addMatch(const char *match, int lineno) {
 }
 
 // Parses the given nmap-service-probes file into the AP class
-static void parse_nmap_service_probe_file(AllProbes *AP, char *filename) {
+/* Must NOT be static because I have externam maintenance tools (servicematch)
+  which use this */
+void parse_nmap_service_probe_file(AllProbes *AP, char *filename) {
   ServiceProbe *newProbe;
   char line[2048];
   int lineno = 0;
@@ -1606,8 +1603,7 @@ ServiceGroup::ServiceGroup(vector<Target *> &Targets, AllProbes *AP) {
       num_hosts_timedout++;
       continue;
     }
-    while((nxtport = Targets[targetno]->ports.nextPort(nxtport, 0, PORT_OPEN,
-			      true))) {
+    while((nxtport = Targets[targetno]->ports.nextPort(nxtport, TCPANDUDP, PORT_OPEN))) {
       svc = new ServiceNFO(AP);
       svc->target = Targets[targetno];
       svc->portno = nxtport->portno;
@@ -1625,8 +1621,7 @@ ServiceGroup::ServiceGroup(vector<Target *> &Targets, AllProbes *AP) {
     if (Targets[targetno]->timedOut(&now)) {
       continue;
     }
-    while((nxtport = Targets[targetno]->ports.nextPort(nxtport, 0, 
-						       PORT_OPENFILTERED, true))) {
+    while((nxtport = Targets[targetno]->ports.nextPort(nxtport, TCPANDUDP, PORT_OPENFILTERED))) {
       svc = new ServiceNFO(AP);
       svc->target = Targets[targetno];
       svc->portno = nxtport->portno;
@@ -1910,6 +1905,7 @@ static int launchSomeServiceProbes(nsock_pool nsp, ServiceGroup *SG) {
   ServiceProbe *nextprobe;
   struct sockaddr_storage ss;
   size_t ss_len;
+  static int warn_no_scanning=1;
 
   while (SG->services_in_progress.size() < SG->ideal_parallelism &&
 	 !SG->services_remaining.empty()) {
@@ -1920,6 +1916,16 @@ static int launchSomeServiceProbes(nsock_pool nsp, ServiceGroup *SG) {
       continue;
     }
     nextprobe = svc->nextProbe(true);
+
+    if (nextprobe == NULL) {
+      if (warn_no_scanning && o.debugging) {
+        printf("Service scan: Not probing some ports due to low intensity\n");
+        warn_no_scanning=0;
+      }
+      end_svcprobe(nsp, PROBESTATE_FINISHED_NOMATCH, SG, svc, NULL);
+      continue;
+    }
+
     // We start by requesting a connection to the target
     if ((svc->niod = nsi_new(nsp, svc)) == NULL) {
       fatal("Failed to allocate Nsock I/O descriptor in launchSomeServiceProbes()");
