@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: service_scan.h 2891 2005-10-01 23:50:27Z fyodor $ */
+/* $Id: service_scan.h 3516 2006-06-19 04:19:52Z fyodor $ */
 
 #ifndef SERVICE_SCAN_H
 #define SERVICE_SCAN_H
@@ -218,7 +218,6 @@ class ServiceProbe {
   ServiceProbe();
   ~ServiceProbe();
   const char *getName() { return probename; }
-  void setName(const char *name); // a copy of name will be made and stored
   // Returns true if this is the "null" probe, meaning it sends no probe and
   // only listens for a banner.  Only TCP services have this.
   bool isNullProbe() { return (probestringlen == 0); }
@@ -341,8 +340,5 @@ public:
    Targets specified. */
 int service_scan(std::vector<Target *> &Targets);
 
-// Parses the given nmap-service-probes file into the AP class
-void parse_nmap_service_probe_file(AllProbes *AP, char *filename);
-
 #endif /* SERVICE_SCAN_H */
 
